;*****************************************************************************************************
;* Programator: David Hankovec DH-servis,vyvoj a vyroba elektroniky http://home.tiscali.cz/dh_servis *                   
;* Program    :	DCF-77 casovy decoder                                   	                       *
;* Datum      : 01-12-2002    											     *
;* Verze 	  : V1.1                  									           *
;Zpracovava signal z prijimace DCF-77. Signal je pripojen na Vami definovatelny bit portu.           *
;Zde je to P3.2 (dcf77_puls). Vystup dat z DCF je po platnem prijmuti telegramu v bytech             *
;sekundy,minuty,hodiny,mesic,rok a den v tydnu ve formatu BCD. Dale je vystup status bitu            *
;synchronni/signalizuje o.k. prijem telegramu									     *
;asynchronni/signalizuje ze neni signal z DCF									     *
;RES_ANT/rezervni antena												     *  
;sekunda/dosazeno sekundy,blika v rytmu 1 sec.								    	     * 
;zimni cas/ nastaven/nulovan podle zimniho/letniho casu							     *
;																     *
;********O rutine******													     *
;Vyuzito casovace 0,externi preruseni vyuzito neni								     *
;Pokud po prvnim zasynchronizovani je cas serizen, bezi uz hodiny samostatne. Veskera data jsou	     *
;o.k. i dny v tydnu. Navic je pridana kontrola dnu v mesici (kolik ma kazdy mesic dni kvuli 	     *
;preteceni) a je pridana kontola prestupneho roku ohledne poctu dni v unoru. Proto jsou		     *
;hodiny schopny kompletne samostatneho chodu. U telegramu z DCF samozrejme kontroluje parity.        *
;*****************************************************************************************************

$MOD52
$TITLE(BYTE SIGNED MULTIPLY)
$PAGEWIDTH(132)
$DEBUG
$OBJECT
$NOPAGING


org 00
AJMP start		;cold START

org 0bH		;INT timer 0
ljmp intt0


;****************************************************************************
;              Deklarace bitovych promennych                                *
;****************************************************************************
dcf77_puls      bit P3.2 ;vstup telegramu DCF
synchronni	bit P1.0	;DCF je synchronni / vystup na LED
asynchronni	bit P1.1	;DCF neni synchronni / vystup na LED
sekunda		bit P1.2	;dosazeno sekundy / vystup na LED
zimni_cas 	bit P1.3	;zimni / letni cas vystup na LED
;tato prirazeni jsou pouze pro vystup na port. V hlavni smycce jsou nastavovana podle priznaku. Proto si tato
;prirazeni muzete jakkoliv zmenit
;*****************************************************************************
       bseg at 30
DATA_BIT:         DBIT  1     ;
DCF_new:      	DBIT  1     ;signalizace, ze bit na vstupu z DCF je zmenen
SYNCHRON:         DBIT  1     ;synchronizace signalem DCF
ASYNC:            DBIT  1     ;0= DCF77-je zasynchronizovano
                              ;1= DCF77-neni synchronni
;----------------------------------------------------------------------------
ok_data:        DBIT  1     ;priznak o.k. nactenych dat z DCF do RAM
RES_ANT:          DBIT  1     ;priznak rezervni anteny
Z_cas:           DBIT  1     ;Zimni / letni cas
PARITY_ERROR:     DBIT  1     ;chyba parity
FULL_SECOND:      DBIT  1     ;1= dokoncena sekunda - cpl bitu po sekunde
FULL_MINUTE:      DBIT  1     ;1= dokoncena minuta (clr in hoofdpr)
;**********************************************************************************
;Predvolba preteceni casovace po 20msec
;Predvolba pro casovac (65535 - 20008) pro 12 MHz krystal

predvolba         equ  0FFFFH - 12000 * 5/3 + 7 + 1  ; vypocet
                              ;12000 pri 12 MHz (delka )
			      ;Start hodnota Timer (= 20msec-Puls)

;****************************************************************************
;Mereni delky(cteni) DCF-77 pulsu: princip = 20 msec * konstanta ,20msec. je preteceni casovace
;rozmezi SS_MIN = 2 * 20msec=40 msec   ////     SS_MAX = 6 * 20msec=120msec
;nastaveni meznich casu trvani pulsu na 1 DCF puls:
 
SS_MIN          equ   02      ;minimalni delka impulsu pro impuls "0" - 2x20=40msec
SS_MAX          equ   06      ;maximalni delka impulsu pro impuls "0" - 6x20=120msec 
SL_MIN          equ   07      ;minimalni delka impulsu pro impuls "1" - 7x20=140msec 
SL_MAX          equ   14      ;minimalni delka impulsu pro impuls "1" - 14x20=280msec

;nastaveni meznich casu trvani mezery na 1 DCF puls:
;princip = 1000msec (50*20msec) - pausovy DCF-77 puls = vysl.

PS_MIN          equ   50 - SL_MAX   ;minimalni delka mezery pro impuls impuls "1" jako vyse,cas je 720msec
PS_MAX          equ   50 - SL_MIN   ;maximalni delka mezery pro impuls impuls "1"	860msec
PL_MIN          equ   50 - SS_MAX   ;minimalni delka mezery pro impuls impuls "0"	880msec	
PL_MAX          equ   50 - SS_MIN   ;minimalni delka mezery pro impuls impuls "0"	960msec

;testovani mezery po 59'sec,zde chybi impuls a mezera trva 2 sekundy
;princip = 2000msec (100*20msec) - nulovy DCF-77 puls

PSD_MIN         equ  100 - SL_MAX   ;minimalni delka mezery pro dvojitou pausu pro posledni puls "1" 1720msec
PSD_MAX         equ  100 - SL_MIN   ;maximalni delka mezery pro dvojitou pausu pro posledni puls "1" 1860msec
PLD_MIN         equ  100 - SS_MAX   ;minimalni delka mezery pro dvojitou pausu pro posledni puls "0" 1880msec	 
PLD_MAX         equ  100 - SS_MIN   ;maximalni delka mezery pro dvojitou pausu pro posledni puls "0" 1960msec

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;Uprava konstant pro jine delky pulsu pri nefunkcnosti rutiny
;POPIS:Pouil jsem originln firemn prijmac DCF urcen k PC s vstupem telegramu DCF, ale procesor se nezasynchronizuje bud vubec, nebo obcas. 
;Po nekolika pokusech jsme doli k zveru, e bude asi nutno ponekud zmenit casovn na rozlien dlky impulsu v rutine. 
;Po zmene ji ve funguje jak m, ale neptejte se mne proc. Sm jsem s upravenm prijmacem dle svho nvodu oivil nekolik kusu hodin bez problmu,
; a ve zmnen prijmac jsem nemel v ruce a nebyla tedy monost "podvat" se osciloskopem na jeho vstup. 
;vice na tomto linku : http://home.tiscali.cz/dalsi/dalsi_hodiny.htm#hod_faq
;SS_MIN equ 03		;minimalni delka impulsu pro impuls "0" - 3x20=60msec 
;SS_MAX equ 07		;maximalni delka impulsu pro impuls "0" - 7x20=140msec 
;SL_MIN equ 08		;minimalni delka impulsu pro impuls "1" - 8x20=160msec
;SL_MAX equ 12		;minimalni delka impulsu pro impuls "1" - 12x20=240msec
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

;****************************************************************************
;                           Definice promennych
;****************************************************************************
T0valuH         equ     high predvolba ;predvolba timer interrupt high
T0valuL         equ     low  predvolba ;predvolba timer interrupt low

;*************************************************************
BLOCK_PTR       equ   50h            ;ukazatel (pointer) pro pohyb v Data-Bloku
DCF_sekundy     equ   51h            ;pocitadlo impulsu do 1 min
COUNTER         equ   52h            ;inkrementovn po 20msec v preruseni od casovace
                                     ;nacita delku DCF-pulsu            
;***************************************************************************
;                     Interni Ram lokace pro real-time-hodiny promenne v BCD
;***************************************************************************
TIKKER         equ   58h            ;pocitadlo preteceni casovace po 20msec. ,toto pocitadlo pretece po 50x cili 1sec.
sekundy        equ   59h            ;platny cas  - sekundy (bud v synchro s DCF nebo samostatne)
minuty         equ   5ah            ;platny cas  - minuty
hodiny         equ   5bh            ;platny cas  - hodiny
den            equ   5ch            ;platny cas  - den
mesic          equ   5dh            ;platny cas  - mesic
rok            equ   5eh            ;platny cas  - rok
d_v_t          equ   5fh            ;platny cas  - den v tydnu

;****************************************************************************
;            Data blok pro rozdelen kompletniho telegramu DCF na jednotlive casti
;****************************************************************************
                      dseg at 60h
DATA_BLOCK:       DS 2            ; Info o zimnim casu,nahr. antene ap. <<<<<<<<<
    DB_MIN:       DS 1            ; minuty informace (+ Parity-Bit P1)   <
    DB_HR:       DS 1            ; hodiny informace    (+ Parity-Bit P2)   <
    DB_DAY:       DS 1            ; den informace                <
    DB_MON:       DS 1            ; den v tydnu/mesic informace              <
    DB_YR:       DS 1            ; rok informace               <<<<<<<<<

;----------------------------------------------------------------------------
;****************************************************************************
;               H L A V N I   S M Y C K A
;****************************************************************************
                        cseg at 50h
;----------------------------------------------------------------------------
start: 
;-------------------------------> inicializace
	
         mov     SP,#7AH         ;nastaveni zasobniku na 80h
        
                        ;cas neni nastaven
      setb RS0  		;druha registrova banka
		call  DCF_INIT          ;inicializace DCF-scan + preruseni timer

    
	MOV sekundy,#0
	MOV minuty,#0
	MOV hodiny,#0
	MOV den,#0
	MOV mesic,#0
	MOV rok,#0
	MOV D_v_t,#0

    
	; disp status bity
status:mov     c,async	
      cpl c
	mov     asynchronni,c
	mov c,synchron    
	cpl c
	mov synchronni,c
      mov     c,Z_cas
	cpl c
	mov zimni_cas,c
      mov     c,full_second
	cpl c   
	mov     sekunda,c
      sjmp status

;********preruseni od casovace************************
INTT0:  CLR RS0		;prvni registrova banka
	  mov     TL0,#T0valuL     ;nacti predvolbu
        mov     TH0,#T0valuH     ;nacti predvolbu
        PUSH    PSW
        PUSH    ACC
        PUSH    DPH
	  PUSH	DPL
	  djnz    tikker,read_dcf
        mov     tikker,#50		;software citac preruseni,po 20msec je incrementovan
        setb    full_second		;50x20 msec = 1 sec.
        call    increment
read_dcf:MOV A,tikker		;po 200 msec. nuluj priznak
	CJNE A,#5,dalsi
	CLR full_second
dalsi:call    scan_dcf77
exit_int:
	pop DPL
	pop DPH
        pop     acc
        pop     psw

	SETB RS0		;druha registrova banka
        reti
;****************************************************************************
;                S u b r u t i n a  D C F 7 7 _  I N I T                  *
; ---------------------------------------------------------------------------
; Inicializace pro cteni DCF-77
; Nastaveni casovace pro vyvolani INT po 20 msec.

DCF_INIT:
         mov     DCF_sekundy,#0     ;nuluj pocitadlo sekund (pulsu) od  DCF_sekundy

;---------------------------------->inicializace timeru
T0INIT:  MOV    TH0,#T0valuH        ;16 bit predvolba v modu 1
         MOV    TL0,#T0valuL
         MOV    TMOD,#11H           ;casovac 0 v modu 1
         SETB   TR0        		;start timer
         SETB   ET0              ;timer 0 INTERUPT ON
         SETB   EA              ;interupts on
;----------------------------------> set/clr  priznaky
         CLR     SYNCHRON           ;nesynchronizovano s DCF
         CLR     ok_data          ;priznak platnych dat v RAM z DCF pro moznost update
         CLR     PARITY_ERROR       ;Neplatna parita
         SETB    ASYNC              ;nesynchronizovano
         CLR     FULL_MINUTE        ;
         CLR     FULL_SECOND        ;
         ret

;****************************************************************************
;         S u b r o u t i n e        S C A N _ D C F 7 7                    *
;****************************************************************************                                                                            |
; Rutina scanuje signal z DCF-77 a zapisuje datove bity postupn do bloku   *
; ("DATA_BLOCK").                                                           *
; --------------- !! Pozor DCF-77 signal je aktivni v nule (negovany) !!-   *
; Ve dvojite pause (59'sec) jsou hodiny synchronizovany s DCF77-signalem.   *
; Pokud jsou vsechna data spravne nactena v bloku, jdi na "UPDATE_TIME".    *
;Tam se zkontroluje parita a pokud je bez chyby nasleduje prepsani dat      *                                           
;----------------------------------------------------------------------------
SCAN_DCF77:
           MOV     C,dcf77_puls      ;prekopirovani urovne ze vstupu
           cpl     c                 ;negovani do spravne urovne
                                     ;na port jde aktivni puls 0, zde potrebujeme 1
           INC     COUNTER           ;citac delky impulsu
           JC      HIGH_NOW
           JNB     DCF_new,TO_THE_END    ;0 ---> 0 : trva uroven nula od minule - nic se nezmenilo,pausa trva
           SJMP    jedna_na_nula            ;1 ---> 0 : nastal prechod z 1 do 0, skok na obsluznou rutinu
HIGH_NOW:  JNB     DCF_new,nula_na_jedna ;0 ---> 1 : nastal prechod z 0 do 1, skok na obsluznou rutinu
TO_THE_END:LJMP    DCF77_RET                ;1 ---> 1 : trva uroven 1 od minule,nic se nezmenilo uroven trva(0 ---> 0)

nula_na_jedna:    ;<----Konec mezery,zacatek pulsu,nova sekunda: ( 0 ---> 1 )
          MOV     DCF_new,C        ;prekopirovani do priznaku, kterym se testuje, zda se nezmenil puls od minule
          MOV     A,COUNTER		;nacti delku pausy
          CLR     C
          SUBB    A,#PS_MIN           ;kontrola minimalni delky pausy
          JC      FAILURE             ;pausa nema minimalni delku - ven
	  SUBB    A,#PS_MAX-PS_MIN+1  ;test pausy pro pausu pro predchozi impuls "1"
          JC      SHORT_PAUSE         ;kratka pausa na data-impuls ( 0/1 )
          SUBB    A,#PL_MIN-PS_MAX-1  ;dvojita pausa na data-impuls ( 0/1 )
          JC      FAILURE
          SUBB    A,#PL_MAX-PL_MIN+1
          JC      LONG_PAUSE
          SUBB    A,#PSD_MIN-PL_MAX-1
          JC      FAILURE
          SUBB    A,#PSD_MAX-PSD_MIN+1
          JC      SHORT_DPAUSE
          SUBB    A,#PLD_MIN-PSD_MAX-1
          JC      FAILURE
          SUBB    A,#PLD_MAX-PLD_MIN+1
          JC      LONG_DPAUSE
          SJMP    FAILURE

LONG_PAUSE:
          SJMP    NEW_PULSE
SHORT_PAUSE:
          JNB     DATA_BIT,FAILURE    ;Signal-Pausa po "1"
          SJMP    NEW_PULSE

SHORT_DPAUSE:
        
          JB      ASYNC,INI_CYCLE     ;Pokud neni zasynchronizovano,inicializace
          JNB     DATA_BIT,FAILURE    ;dvojita pausa na hodnotu "1" na konci tlg.
          SJMP    CHECK_58
LONG_DPAUSE:
          JB      ASYNC,INI_CYCLE     ;Pokud neni zasynchronizovano,inicializace
          JB      DATA_BIT,FAILURE    ;dvojita pausa na hodnotu  "0" na konci tlg.
CHECK_58:
          mov     r1,DCF_sekundy
          CJNE    r1,#58,FAILURE      ;Jde o 58. Sekundu ??, ne chyba,ano pokracuj

	      CALL    UPDATE_TIME         ;<--- volani update dat, prepise aktualni data daty z DCF
          SETB    FULL_MINUTE         
          SETB    SYNCHRON		;nastav priznak synchronniho DCF a hodin
          SETB    ok_data           ;nastav priznak plneho bloku RAM, pokud je chyba v parite,stejne je nastaven                                 

 MOV     A,DATA_BLOCK+1
          MOV     C,ACC.5             ;= letni cas
          MOV     Z_cas, C           ;                                 
          MOV     C, ACC.3            ;= rezervni antena
          MOV     RES_ANT, C          ;

INI_CYCLE:MOV     DCF_sekundy,#-1     ;Pripraveno pro dalsi cyklus (minutu)
          CLR     ASYNC               ;nuluj priznak asynchronni

NEW_PULSE:INC     DCF_sekundy         ;novy platny impuls,zvys pocitadlo sekund od DCF
 
         SJMP    DCF77_END
;----------------------------------------------------------------------------
FAILURE:  CLR     SYNCHRON            ;vstupni impuls od DCF je mimo toleranci
          SETB    ASYNC		;nastav priznak asynchronni, nuluj synchronni
          SJMP    DCF77_END
;----------------------------------------------------------------------------

jedna_na_nula:    ;<-------Dekodovani impulsu podle delky:( 1 --> 0 )
          MOV     DCF_new,C        ;aktualizuj
          MOV     A,COUNTER
          CLR     C
          SUBB    A,#SS_MIN           ;Odecteni minimalniho trvani pulsu
          JC      FAILURE             ;pokud byl kratsi, neni platny 
          SUBB    A,#SS_MAX-SS_MIN+1  ;2 moznosti:
          JC      SHORT_PULSE         ;0.1 sec Impuls  =  "0"
          SUBB    A,#SL_MIN-SS_MAX-1  ;0.2 sec Impuls  =  "1"
          JC      FAILURE
          SUBB    A,#SL_MAX-SL_MIN+1
          JNC     FAILURE

LONG_PULSE:SETB   DATA_BIT            ;Hodnota bitu je '1'
          SJMP    BIT_KNOWN
SHORT_PULSE:CLR   DATA_BIT            ;Hodnota bitu je '0'

BIT_KNOWN:

          JB      ASYNC,DCF77_END     ;Neni synchro: VEN!
          mov     r1,DCF_sekundy
          CJNE    r1,#00,ADD?         ;Je ukoncen cyklus ??
          JB      DATA_BIT,FAILURE    ;Pokud neni ukoncen a data bit ma hodnotu 1 je to chyba
          SJMP    DCF77_END
ADD?:
          mov     r1,DCF_sekundy
          CJNE    r1,#04,ADD_IN?      
          MOV     block_ptr,#DATA_BLOCK
          mov     r0,block_ptr
          MOV     @r0,#80H            
          SJMP    SHIFT_IN
ADD_IN?:  JC      DCF77_END           
          mov     r1,DCF_sekundy
          CJNE    r1,#20,PAR?         
          JNB     DATA_BIT,FAILURE    
          SJMP    DCF77_END
PAR?:     mov     r1,DCF_sekundy
          CJNE    r1,#58,SHIFT_IN     
          SJMP    DCF77_END           
SHIFT_IN:
          MOV     C,DATA_BIT          ;
          mov     r0,block_ptr        ;
          XCH     A,@r0               
          RRC     A                   
          XCH     A,@r0
          JNC     DCF77_END           ;
	      MOV     A,#10000000B        ;mezni hodnota 8-bit hodnota
HR?:      mov     r1,DCF_sekundy
          CJNE    r1,#28,den?         
          RR      A                   ;mezni hodnota 7-bit hodnota
den?:     CJNE    r1,#35,NEXT_ENTRY
          RR      A
	    RR      A                   ;mezni hodnota 6-bit hodnota
NEXT_ENTRY:
          INC     block_ptr			;incrementuj ukazatel
          mov     r0,block_ptr		
          MOV     @r0,A              ;prepis udaj novou hodnotou

DCF77_END:MOV     COUNTER,#00        ;nuluj citac obehu
          

DCF77_RET:
          RET                        ;navrat
;****************************************************************************
;  S u b r o u t i n e        U P D A T E _ T I M E                         *
;****************************************************************************                                                                            
;  Rutina dekoduje "DATA_BLOCK" z casovych registru.
;  Vysledek je po kontrole parity zapsan do odpovidajicich byte.
; Pokud se vyskytne jedna chyba, nic neni prepsano a zustava puvodni udaj!!         
;----------------------------------------------------------------------------

UPDATE_TIME:      ;<---------------- nejprve kontroluje paritu

          MOV     R3,#05              ;casovy format= 5 Byte z "DATA_BLOCK"
          MOV     BLOCK_PTR,#DATA_BLOCK + 1
          MOV     B,#00               ;B: Parity-Bity 
GET_PAR:  INC     BLOCK_PTR
          mov     r0,block_ptr
          MOV     A,@r0
          MOV     C,PSW.0             
          XCH     A,B
          RLC     A
          XCH     A,B                 ;<---- porovnani
          DJNZ    R3,GET_PAR
          MOV     A,B
          MOV     C,DATA_BIT          
          RLC     A
          JNB     PSW.0,DATA_TO_REGS  ;Total Parity je OK
          SETB    PARITY_ERROR        ;                            
          SJMP    UPDATE_END          ;Parity-chyba -> VEN!

DATA_TO_REGS:     ;<------------------;parita O.K. prepis data.

	 MOV     A,DB_MIN            ;minutova-informace
          CLR     ACC.7               ;Paritni-Bit pryc
          MOV     minuty,A

          MOV     A,DB_HR             ;hodiny informace
	      RR      A                   ;7-data-bit hodnota
	      CLR     ACC.6               ;Paritni-Bit pryc
           MOV     hodiny,A

          MOV     A,DB_DAY            ;datum den-informace
          RR      A
	      RR      A                   ;6-data-bit hodnota
          MOV     den,A

          MOV     A,DB_MON            ;den v tydnu/mesic informace
          MOV     B,#00001000B        ;rozdel: den v tydnu = 3 Bit
          DIV     AB                  ;A= mesic    B=d_v_t
          MOV     d_v_t,B
          MOV     mesic,A

          MOV     A,DB_YR             ;rok-informace
          MOV     rok,A

          MOV     sekundy,#99H        ;<-- sekundy=0 
          MOV     tikker,#04          ;real time z DCF77-aktualizuj.
                                      ;Timer-citac = 40 msec.
          CLR     PARITY_ERROR        ;zadna chybna parita

UPDATE_END: RET

;****************************************************************************
;            S u b r u t i n a        I N C R E M E N T                   *   
;****************************************************************************
;   Rutina realneho casu sec:min:hr den/DvT:mesic:rok )
;   Kontroluje a spravne uklada pocet dni v mesici (30,31) a navic kontroluje
;   prestupny rok a upravuje unor. Bezi samostatne i bez synchro z DCF
;----------------------------------------------------------------------------

INCREMENT:
	MOV A,sekundy		;nacti do ACC byte vteriny
	ADD A,#1H			;pricti 1
	DA A				;korekce
	MOV sekundy,A		;uloz vteriny zpet
	XRL A,#60H			;je dosazeno minuty ?	
	JNZ INCR_END		;neni pak navrat z preruseni
	MOV sekundy,A		;nuluj byte vteriny
	SETB    FULL_MINUTE
	MOV A,minuty		;nacti do ACC obsah byte minuty
	ADD A,#1			;pricti k ACC 1
	DA A				;korekce
	MOV minuty,A		;uloz minuty zpet
	XRL A,#60H			;je dosazeno hodiny ?
	JNZ INCR_END			;neni pak navrat z preruseni pres test casu z tabulky
	MOV minuty,A		;nuluj byte minuty
	MOV A,hodiny		;nacti do ACC obsah byte hodiny
	ADD A,#1			;pricti k ACC 1
	DA A				;korekce
	MOV hodiny,A		;uloz zpet hodiny
	XRL A,#24H			;je dosazeno 24 hodin ?
	JNZ INCR_END			;neni pak navrat z preruseni pres test casu z tabulky
	MOV hodiny,A		;nuluj byte hodiny

          INC     d_v_t
          mov     r7,d_v_t
          CJNE    r7,#08,NEXT_DAY
	      MOV     d_v_t,#01         ;po preteceni dni pres 7 znovu 1-pondeli

NEXT_DAY: 
MOV A,den		;nacti do ACC obsah byte mesic
	ADD A,#1			;pricti k ACC 1
	DA A				;korekce
	MOV den,A		;uloz zpet mesic
      MOV     DPTR,#dnu_na_mesic-1
      MOV     A,mesic             ;mesic(1...12)+TAB-1 = nastaveni
	MOVC    A,@A+DPTR           ;delka mesice / pocet dni
      MOV     r5,mesic
      CJNE    r5,#02,COMP_LEN
      MOV     B,A                 ;<-- mesic = unor
      MOV     A,rok		;cti rok
          
	;tady prevest rok z BCD na binarni
	LCALL BCD_2_HEX
	ANL     A,#00000011B			;zjisti zda je prestupny
      JNZ     NORMAL              ;pokud neni nula / neni prestupny
      MOV A,B		;je prestupny unor= 29 dni / zvys pocet dni z tabulky o jeden / pro unor 
	ADD A,#1
	DA A
	MOV B,A

NORMAL:   MOV     A,B                 ;neni prestupny, vrat pocet dni v mesici zpet
COMP_LEN: MOV     B,den			;cti aktualni den
          CJNE    A,B,INCR_END	;porovnej a pri preteceni nuluj
          MOV     den,#01H
        
	MOV A,mesic		;nacti do ACC obsah byte mesic
	ADD A,#1			;pricti k ACC 1
	DA A				;korekce
	MOV mesic,A		;uloz zpet mesic
	XRL A,#13H			;je dosazeno 12 mesicu ?
	JNZ INCR_END			;neni pak navrat 
	MOV mesic,#01H		;byte mesic / leden

	MOV A,rok		;nacti do ACC obsah byte roky
	ADD A,#1			;pricti k ACC 1
	DA A				;korekce
	MOV rok,A		;uloz zpet roky
	XRL A,#100H			;je dosazeno 100 let ?
	JNZ INCR_END			;neni pak navrat 
	MOV rok,A		;nuluj byte roky
INCR_END: RET
;--------------------------------------------------------------------------------
BCD_2_HEX:
		MOV R7,B			
		PUSH	ACC			
		ANL	A,#0f0h		
		SWAP	A			
		MOV	B,#10			
		MUL	AB			
		MOV	B,A			
		POP	ACC			
		ANL	A,#00fh		
		ADD	A,B			
		MOV B,R7			
		RET
;------------------------------------------------------------------------------

dnu_na_mesic:                       ;Delka mesice - pocet dni,mesice jsou v BCD,ale vyhledavani v tabukce je hexa
          DB 32h                      ;cili mesic napr. 12BCD odpovida 18h a na te pozici je odpovidajici pocet dni
          DB 29h                       
          DB 32h
          DB 31h,32h,31h
          DB 32h,32h,31h,0,0,0,0,0,0,32h,31h,32h
         

 END
